<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\COZ\Services\Pla10500Service;
use Teknisa\COZ\Services\GeracaoProdutoAcabadoService;
use Teknisa\COZ\Services\Pla10500RemoteRepository;

class Pla10500Controller extends \Teknisa\GER\Controllers\TeknisaCrud
{

    protected $novocodigo;
    protected $crudService;
    protected $parameterBag;
    protected $pla10500service;
    protected $geracaoProdutoAcabadoService;
    protected $entityManager;
    protected $dataSourceManager;
    protected $pla10500remoteRepository;
    protected $util;

    public function __construct(
        $crudService, $parameterBag, $novocodigo, Pla10500Service $pla10500service,
        GeracaoProdutoAcabadoService $geracaoProdutoAcabadoService, $entityManager, $dataSourceManager, Pla10500RemoteRepository $pla10500remoteRepository,
        $util
    )
    {
        parent::__construct($crudService, $parameterBag);
        $this->novocodigo = $novocodigo;
        $this->pla10500service = $pla10500service;
        $this->geracaoProdutoAcabadoService = $geracaoProdutoAcabadoService;
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->pla10500remoteRepository = $pla10500remoteRepository;
        $this->util = $util;
    }

    public function filtraPlanejamento(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();

        $row['CDFILIAL'] = $row['CDCENTRAL'];

        if ($row['NOVOPLANPROD'] === 'S') {
            // Gera Planejamento
            $planproducao = $this->pla10500service->geraPlanejamento($row);
            $nrplanprod = $planproducao["P_RETORNO"];

            if (empty($nrplanprod)) {
                $message = new Message(
                    'Não foi possível gerar Planejamento de Produção!', 'W'
                );
                return $response->addMessage($message);
            } else {
                $message = new Message(
                    'Planejamento de Produção gerado com sucesso!', 'M'
                );
                $response->addMessage($message);
            }
        } else {
            // Consulta Lançamento
            $existeLancamento = $this->pla10500service->retornaPlanejamento(
                $row
            );

            if (empty($existeLancamento)) {
                $message = new Message(
                    'Planejamento de Produção não tem lançamento!', 'W'
                );
                return $response->addMessage($message);
            } else {
                /*if($existeLancamento['IDRETIRADA'] == 'S'){
                  $message = new Message('Planejamento de Produção está bloqueado!', 'W');
                  $response->addMessage($message);
                }*/
                $nrplanprod = $row['NRPLANPROD'];
            }
        }

        $this->salvaDescricaoPlanejamento(
            $row['NRORG'], $row['CDCENTRAL'], $nrplanprod, $row['DSPLANEJPRODUCAO']
        );

        $response->setDataSets(
            array($this->makeSingleValueDataset('NRPLANPROD', $nrplanprod))
        );
    }

    public function salvaDescricaoPlanejamento(
        $nrorg, $central, $planejamento, $descricao
    ) {
        $param = array(
            "NRORG" => $nrorg,
            "CDFILIAL" => $central,
            "NRPLANPROD" => $planejamento,
            "DSPLANEJPRODUCAO" => $descricao,
            "__is_new" => false
        );

        $requestInser = new DTO\Request\Row($param, '', '', '');
        $dadosInserir = $this->crudService->getRowsToSave(
            $requestInser, "planproducao"
        );

        $this->crudService->save("planproducao", $dadosInserir);
    }

    public function cancelaPlanejamento(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();
        try {
            $this->pla10500service->cancelaPlanejamento((array)$row);
            $message = new Message(
                'Planejamento de Produção cancelado com sucesso!', 'M'
            );
            $response->addMessage($message);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function retornaPlanejamento(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();
        try {
            $response->addDataSet(
                new DataSet(
                    'planprod',
                    $this->pla10500service->retornaPlanejamento($row)
                )
            );
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function retornaValoresProdutoNovaReceita(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();
        try {
            
            $fieldsToFilter = array("CDFILIAL", "CDPRODUTO", "QTPRODRECEPRAT", "DTCONSUMO");
            $paramObjectArray = $this->buildFilterArrayFromRow((array)$row, $fieldsToFilter);
            $valoresproduto = $this->pla10500remoteRepository->fetchValoresProdutoNovaReceita($paramObjectArray);

            $response->addDataSet(
                new DataSet(
                    'valoresprodutonovareceita',$valoresproduto
                )
            );
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function buildFilterArrayFromRow($row, $filterKeys){

        //pega todos os valores de $row com chaves presentes em $filterKeys
        //o array_flip faz com que os valores de $filterKeys sejam chaves
        $paramValueArray = array_intersect_key($row, array_flip($filterKeys));

        return array_map(function($key, $value){
            return array("columnName"=>$key, "operator"=>"=", "value"=>$value);
        }, array_keys($paramValueArray), $paramValueArray);
    }

    public function retornaTipopratoNovaReceita(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();
        try {
            $response->addDataSet(
                new DataSet(
                    'tipopratonovareceita',
                    $this->pla10500service->retornaTipopratoNovaReceita($row)
                )
            );
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function atualizaPlanprodfiliagrega(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();
        try {
            $this->pla10500service->atualizaPlanprodfiliagrega($row);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function atualizaPlanprodacabado(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();
        try {
            $this->pla10500service->atualizaPlanprodacabado($row);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function verificaEstoqueProdutosReceita(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();

        try {
            $nrorg = $row['NRORG'];
            $produtos = $row['PRODUTOS'];
            $central = $row['CENTRAL'];
            $nrPlanprod = $row['NRPLANPROD'];
            $data = $row['DATA'];
            $produzir = $row['IDPRODUZIR'];
            $bloqueia = $row['IDBLOQUEIA'];
            $origem = $row['ORIGEM'];
            $operador = $row['CDOPERADOR'];
            $strProdutos = $row['STRPRODUTO'];

            if ($produzir === 'S' and $bloqueia === 'S') {
                if ($produtos) {
                    $lista
                        = $this->geracaoProdutoAcabadoService->montaListaProdutoAcabado(
                        $nrorg, $produtos
                    );
                    if ($origem === 'PLANEJAMENTO') {
                        $prodSaldoInsuf
                            = $this->pla10500service->verificaEstoqueProdutosReceitaComReserva(
                            $nrorg, $central, $nrPlanprod, $lista, $data
                        );
                    } else {
                        $prodSaldoInsuf
                            = $this->pla10500service->verificaEstoqueProdutosReceita(
                            $nrorg, $central, $nrPlanprod, $lista, $data
                        );
                    }

                    if ($prodSaldoInsuf) {
                        if (array_key_exists('JUSTIFICATIVA', $row)
                            and $row['JUSTIFICATIVA']
                        ) {
                            $this->pla10500service->atualizaLogMP(
                                $nrorg, $row['NRLOGMATPRIMA'],
                                $row['JUSTIFICATIVA']
                            );
                        } else {
                            $codigo = $this->novocodigo->getCodeByTableName(
                                "LOGMATPRIMA", $nrorg
                            );

                            $this->pla10500service->salvaLogMP(
                                $nrorg, $nrPlanprod, $codigo, $origem,
                                $strProdutos,
                                $prodSaldoInsuf, $operador
                            );

                            $response->addDataSet(
                                new DataSet(
                                    'PRODSALDOINSUF', array($codigo)
                                )
                            );
                            return;
                        }
                    }
                }
            }

            $response->addDataSet(new DataSet('PRODSALDOINSUF', array(false)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function atualizaPaiProduzirProdacabado(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();
        try {
            $this->pla10500service->atualizaPaiProduzirProdacabado($row);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function atualizaFilhoProduzirProdacabado(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();
        try {
            $this->pla10500service->atualizaFilhoProduzirProdacabado($row);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function atualizaFilhoProdFiliAgrega(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();
        try {
            $this->pla10500service->atualizaFilhoProdFiliAgrega($row);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function retornaFilhosProduzidos(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();
        try {
            $response->addDataSet(
                new DataSet(
                    'produzidos',
                    $this->pla10500service->retornaFilhosProduzidos($row)
                )
            );
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function retornaProdutoJaProduzido(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();
        try {
            $response->addDataSet(
                new DataSet(
                    'produzido',
                    $this->pla10500service->retornaProdutoJaProduzido($row)
                )
            );
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function retornaProdutosProduzidos(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();
        try {
            $response->addDataSet(
                new DataSet(
                    'produzido',
                    $this->pla10500service->retornaProdutosProduzidos($row)
                )
            );
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function makeSingleValueDataset($column, $value)
    {
        return new DataSet(
            $column, array('columnName' => $column, 'value' => $value)
        );
    }

    public function proceAlteraQtdeProdacabado(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();

        try {
            $row['NRJUSTPLANPROD'] = $this->pla10500service->insereJustPlanProd(
                $row['DSJUSTIFICA']
            );
            if($row['QTAPRODUZIR'] == 0) $row['INSEREDIF'] = 'N';
            $possuiaCapacidade = $this->pla10500service->proceAlteraQtdeProdacabado($row);
            $response->addDataset(new DataSet('possuiaCapacidade', $possuiaCapacidade));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function geraPedidoProducao($request,$response)
    {
        $row = $request->getRow();
        try {
            $nrlogmatprima = $this->pla10500service->geraPedidoProducao((array)$row);
            $data = $this->findLogmatprima($nrlogmatprima, $row);
            $response->addDataSet(new DataSet('pedidoproducao', $data));
            
            $arrayDelete = array("NRLOGMATPRIMA"=>$nrlogmatprima['P_NRLOGMATPRIMA']);
            $requestDelete = new DTO\Request\Row($arrayDelete, '', '', '');
            $this->excluiLogMatprima($requestDelete, $response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function findLogmatprima($nrlogmatprima, $row){
        $connection = $this->entityManager->getConnection();
        $query="SELECT P.CDARVPROD, LM.CDPRODUTO, F.NMFILIAL, 
            LM.NRORG, LM.NMPRODUTO, 
            LM.CDFILIAL, LM.CDSERVFILI, 
            S.NMSERVICO, LM.DTCARDAPIO 
        FROM LOGMATPRIMA LM 
        INNER JOIN SERVICO S ON 
            LM.CDFILIAL = S.CDFILIAL AND 
            LM.CDSERVFILI = S.CDSERVFILI AND 
            LM.NRORG = S.NRORG 
        INNER JOIN FILIAL F ON 
            LM.CDFILIAL = F.CDFILIAL AND 
            LM.NRORG = F.NRORG
        INNER JOIN PRODUTO P ON P.CDPRODUTO = LM.CDPRODUTO
        WHERE NRLOGMATPRIMA = :NRLOGMATPRIMA AND 
              NRPLANPROD = :NRPLANPROD AND 
              DSORIGEM = 'PEDIDO'
        ORDER BY LM.CDFILIAL, LM.CDSERVFILI, P.CDARVPROD";
              
        $data = $connection->fetchAll($query, array("NRLOGMATPRIMA"=>$nrlogmatprima['P_NRLOGMATPRIMA'], "NRORG"=>$row['NRORG'], "NRPLANPROD"=>$row['NRPLANPROD']));
        return $data;
    }

    public function proceLiberaProducao(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();

        try {
            $response->addDataSet(
                new DataSet(
                    'produzido',
                    $this->pla10500service->proceLiberaProducao($row)
                )
            );
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function atualizaLogMatprima(
        DTO\Request\Row $request, DTO\Response $response
    )
    {
        $row = $request->getRow();
        try {
            $this->pla10500service->atualizaLogMP(
                $row['NRORG'], $row['NRLOGMATPRIMA'],
                $row['JUSTIFICATIVA']
            );
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function excluirMatPrimConsol(
        DTO\Request\Row $request, DTO\Response $response
    ){
        try {
            $retorno = array();
            $rows = $request->getRow();
            foreach ($rows as $row) {
                $conditions = array (
                    array (
                        "columnName" => "NRORG",
                        "operator" 	 => "=",
                        "value" 	 => $this->crudService->getEnvironment()->getNrOrg()
                    ),
                    array (
                        "columnName" => "NRPLANPROD",
                        "operator" 	 => "=",
                        "value" 	 => $row['NRPLANPROD']
                    ),
                    array (
                        "columnName" => "CDPRODUTO",
                        "operator" 	 => "=",
                        "value" 	 => $row['CDPRODUTO']
                    )
                );
                $dataSet = $this->crudService->getFilteredDataSet('pla07000_receita_por_agregada', $conditions);
                $result = $dataSet->getRows();
                if(!empty($result)){
                    $dataSet = new DataSet('planrecprod', $result);
                    $this->crudService->delete($response, $dataSet);
                    $retorno[]= $result;
                } else {
                    $retorno = array(false);
                }
            }

            $response->addDataset(new DataSet('planrecprod', $retorno));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }


    public function salvaMatPrimConsol(
        DTO\Request\Row $request, DTO\Response $response
    ){
        try {
            $row = $request->getRow();

            $nrorg = $this->crudService->getEnvironment()->getNrOrg();
            $data = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
            $operador = $this->crudService->getEnvironment()->getCdOperador();

            $nrPlanprod = $row['NRPLANPROD'];
            $valorUnit = $row['VRUNITRECEPRAT'];
            $qtInserida = $qtProduto = $row['QTAPRODUZIR'];
            $produtoNovo = $row['CDPRODUTO'];
            $T_paSelecionado = $row['T_PRODUTOACABADO'];
            $paSelecionado = $row['PRODUTOACABADO'];
            $qtExtra = $row['QTEXTRA'];
            $qtInsExtra = $row['QTEXTRA'];
            $qtDevolvida = $row['QTDEVOLVIDA'];
            $qtInsDevolvida = $row['QTDEVOLVIDA'];
            $qtLiberada =$row['QTLIBERADA'];
            $totalReceitas = 0;

            $conditions = array();
            if ($T_paSelecionado === 'S') {
                $conditions[] = array (
                    array (
                        "columnName" => "NRORG",
                        "operator"   => "=",
                        "value"      => $nrorg
                    ),
                    array (
                        "columnName" => "NRPLANPROD",
                        "operator"   => "=",
                        "value"      => $nrPlanprod
                    )
                );
            }
            else {
                $numPaSelecionados = 0;
                foreach ($paSelecionado as $pa) {
                    $conditions[] = array (
                        array (
                            "columnName" => "NRORG",
                            "operator"   => "=",
                            "value"      => $nrorg
                        ),
                        array (
                            "columnName" => "NRPLANPROD",
                            "operator"   => "=",
                            "value"      => $nrPlanprod
                        ),
                        array (
                            "columnName" => "CDPRODACABADO",
                            "operator"   => "=",
                            "value"      => $pa
                        )
                    );

                    $numPaSelecionados++;
                }

                $qtProduto = round($qtProduto/$numPaSelecionados, 5);
            }

            foreach ($conditions as $condition) {
                $dataSet = $this->crudService->getFilteredDataSet(
                    'pla07000_receita_por_agregada', $condition
                );
                $planRecProd = $dataSet->getRows();

                if (!empty($planRecProd)) {
                    $linhaNova = array();
                    $produtoAcabado
                        =
                    $central = $dtConsumo = $agregada = $servAgregada = '';
                    $tipoPrato = $coockChill
                        = $finalizacao = $tipo = $estado = $sequencia = '';

                    foreach ($planRecProd as $receita) {
                        if ($produtoAcabado <> $receita['CDPRODACABADO'] OR
                            $central <> $receita['CDFILIAL'] OR
                            $dtConsumo <> $receita['DTCONSUMO'] OR
                            $agregada <> $receita['CDFILIAGREG'] OR
                            $servAgregada <> $receita['CDSERVFILIAGREG'] OR
                            $tipoPrato <> $receita['CDTIPOPRAT'] OR
                            $coockChill <> $receita['IDCOOKCHILL'] OR
                            $finalizacao <> $receita['IDFINALIZACAO'] OR
                            $tipo <> $receita['IDTPPRODCHILL'] OR
                            $estado <> $receita['IDESTADO'] OR
                            $sequencia <> $receita['NRSEQPRODACABADO']
                        ) {
                            $totalReceitas++;
                            $linhaNova[] = array(
                                "NRPLANPROD" => $nrPlanprod,
                                "CDPRODACABADO" => $receita['CDPRODACABADO'],
                                "CDFILIAL" => $receita['CDFILIAL'],
                                "DTCONSUMO" => $receita['DTCONSUMO'],
                                "CDFILIAGREG" => $receita['CDFILIAGREG'],
                                "CDSERVFILIAGREG" => $receita['CDSERVFILIAGREG'],
                                "CDPRODUTO" => $produtoNovo,
                                "CDTIPOPRAT" => $receita['CDTIPOPRAT'],
                                "QTPRODRECEPRAT" => 0,
                                "VRUNITRECEPRAT" => $valorUnit,
                                "VRTOTALRECEPRAT" => 0,
                                "IDTPRECEITA" => $receita['IDTPRECEITA'],
                                "IDADIANTADO" => $receita['IDADIANTADO'],
                                "NRSEQITRETI" => '',
                                "NRORG" => $nrorg,
                                "DTINCLUSAO" => $data,
                                "DTULTATU" => $data,
                                "NRORGINCLUSAO" => $nrorg,
                                "CDOPERINCLUSAO" => $operador,
                                "NRORGULTATU" => $nrorg,
                                "CDOPERULTATU" => $operador,
                                "IDATIVO" => 'S',
                                "IDCOOKCHILL" => $receita['IDCOOKCHILL'],
                                "IDFINALIZACAO" => $receita['IDFINALIZACAO'],
                                "IDTPPRODCHILL" => $receita['IDTPPRODCHILL'],
                                "IDESTADO" => $receita['IDESTADO'],
                                "NRSEQPRODACABADO" => $receita['NRSEQPRODACABADO'],
                                "QTLIBERADA" => $qtLiberada,
                                "QTAPRODUZIR" => 0,
                                "QTEXTRA" => 0,
                                "QTDEVOLVIDA" => 0,
                                "__is_new" => true
                            );
                        }

                        $produtoAcabado = $receita['CDPRODACABADO'];
                        $central = $receita['CDFILIAL'];
                        $dtConsumo = $receita['DTCONSUMO'];
                        $agregada = $receita['CDFILIAGREG'];
                        $servAgregada = $receita['CDSERVFILIAGREG'];
                        $tipoPrato = $receita['CDTIPOPRAT'];
                        $coockChill = $receita['IDCOOKCHILL'];
                        $finalizacao = $receita['IDFINALIZACAO'];
                        $tipo = $receita['IDTPPRODCHILL'];
                        $estado = $receita['IDESTADO'];
                        $sequencia = $receita['NRSEQPRODACABADO'];
                    }

                    $qtAProduzir = round($qtProduto / $totalReceitas, 5);
                    $newQtExtra = round($qtExtra / $totalReceitas, 5);
                    $newQtDevolvida = round($qtDevolvida / $totalReceitas, 5);
                    $valorTotal = ($qtAProduzir * $valorUnit);
                    $linhaInsercao = array();
                    foreach ($linhaNova as $linha) {
                        $linha["QTAPRODUZIR"] = $qtAProduzir;
                        $linha["QTEXTRA"] = $newQtExtra;
                        $linha["QTDEVOLVIDA"] = $newQtDevolvida;
                        $linha["VRTOTALRECEPRAT"] = $valorTotal;
                        $linha["OPERACAO"] = 1;
                        $linhaInsercao[] = $linha;
                        $qtInserida -= $qtAProduzir;
                        $qtInsExtra -= $newQtExtra;
                        $qtInsDevolvida -= $newQtDevolvida;
                    }

                    //Valida a qtd total inserida e atualiza o ultimo se precisar
                    if ($qtInserida <> 0) {
                        $ultimo = end($linhaInsercao);
                        array_pop($linhaInsercao);
                        $ultimo["QTAPRODUZIR"] += round($qtInserida, 5);
                        $ultimo["QTEXTRA"] += round($qtInsExtra, 5);
                        $ultimo["QTDEVOLVIDA"] += round($qtInsDevolvida, 5);
                        $linhaInsercao[] = $ultimo;
                    }

                    if (count($linhaInsercao) > 0) {
                        $this->crudService->save('planrecprod', $linhaInsercao);
                        $this->pla10500service->atualizaPlanprodfiliagrega(
                            $linhaInsercao
                        );
                    }
                }
            }

            $response->addDataset(new DataSet('planrecprod', array($totalReceitas)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }


    public function editaMatPrimConsol(
        DTO\Request\Row $request, DTO\Response $response
    ){
        try {
            $row = $request->getRow();
            $nrorg = $this->crudService->getEnvironment()->getNrOrg();
            $data = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
            $operador = $this->crudService->getEnvironment()->getCdOperador();
            
            $nrPlanprod = $row['NRPLANPROD'];
            $produtoNovo = $row['CDPRODUTO'];
            $qtAprodConc = $this->getQtAproduzirConc($nrPlanprod, $produtoNovo);
            
            $valorUnit = $row['VRUNITRECEPRAT'];
            $qtInserida = $row['QTAPRODUZIR'] - $qtAprodConc;
            $qtProduto = $row['QTAPRODUZIR'] - $qtAprodConc;
            $qtProdutoAnterior = ($row['QTAPRODUZIR_ANT'] - $qtAprodConc) <= 0 ? 1 : ($row['QTAPRODUZIR_ANT'] - $qtAprodConc);
            $qtInsExtra = $row['QTEXTRA'];
            $qtExtra = $row['QTEXTRA'];
            $qtInsDevolvida = $row['QTDEVOLVIDA'];
            $qtDevolvida = $row['QTDEVOLVIDA'];
            $produtoAnterior = $row['CDPRODUTO_ANT'];
            $T_paSelecionado = $row['T_PRODUTOACABADO'];
            $qtLiberada =$row['QTLIBERADA']; 
            
            $conditions = array();
            if ($T_paSelecionado === 'S') {
                $conditions[] = array (
                    array (
                        "columnName" => "NRORG",
                        "operator"   => "=",
                        "value"      => $nrorg
                    ),
                    array (
                        "columnName" => "NRPLANPROD",
                        "operator"   => "=",
                        "value"      => $nrPlanprod
                    ),
                    array (
                        "columnName" => "CDPRODUTO",
                        "operator"   => "=",
                        "value"      => $produtoAnterior
                    )
                );
            }
            else {
                $paSelecionado = $row['PRODUTOACABADO'];
                foreach ($paSelecionado as $pa) {
                    $conditions[] = array (
                        array (
                            "columnName" => "NRORG",
                            "operator"   => "=",
                            "value"      => $nrorg
                        ),
                        array (
                            "columnName" => "NRPLANPROD",
                            "operator"   => "=",
                            "value"      => $nrPlanprod
                        ),
                        array (
                            "columnName" => "CDPRODUTO",
                            "operator"   => "=",
                            "value"      => $produtoAnterior
                        ),
                        array (
                            "columnName" => "CDPRODACABADO",
                            "operator"   => "=",
                            "value"      => $pa
                        )
                    );
                }
            }

            $registrosAlterados = 0;
            foreach ($conditions as $condition) {
                $dataSet = $this->crudService->getFilteredDataSet(
                    'pla07000_receita_por_agregada', $condition
                );

                $planRecProd = $dataSet->getRows();
                $qntMatPrimas = sizeof($planRecProd);
                
                if (!empty($planRecProd)) {
                    $linhaNova = array();
                    $totalProdReceprat = $this->totalProdReceprat($planRecProd);
                    
                    foreach ($planRecProd as $receita) {
                        if($qtLiberada == 0){
                            $proporcao = 0;
                            $qtAProduzir = $receita['QTAPRODUZIR'];
                        }else {
                            if($receita['QTAPRODUZIR'] > 0){
                                $proporcao = round($receita['QTAPRODUZIR'] / $qtProdutoAnterior, 5);
                                $qtAProduzir = round($qtProduto * $proporcao, 5);
                            }else{
                                $proporcao = round($receita['QTPRODRECEPRAT'] / $totalProdReceprat , 5);
                                $qtAProduzir = round($qtProduto * $proporcao, 5);    
                            }
                       }
                        $qtProduto -= $qtAProduzir; 

                        $newQtExtra = round($qtExtra * $proporcao, 5);
                        $qtInsExtra -= $newQtExtra;
                        
                        $newQtDevolvida = round($qtDevolvida * $proporcao,5);
                        $qtInsDevolvida -= $newQtDevolvida;
                        
                        $valorTotal = $qtAProduzir * $valorUnit;
                        $linhaNova[] = array(
                            "NRPLANPROD" => $receita['NRPLANPROD'],
                            "CDPRODACABADO" => $receita['CDPRODACABADO'],
                            "CDFILIAL" => $receita['CDFILIAL'],
                            "DTCONSUMO" => $receita['DTCONSUMO'],
                            "CDFILIAGREG" => $receita['CDFILIAGREG'],
                            "CDSERVFILIAGREG" => $receita['CDSERVFILIAGREG'],
                            "CDPRODUTO" => $produtoNovo,
                            "CDTIPOPRAT" => $receita['CDTIPOPRAT'],
                            "QTPRODRECEPRAT" => $receita['QTPRODRECEPRAT'],
                            "QTAPRODUZIR" => $qtAProduzir,
                            "QTEXTRA" => $newQtExtra,
                            "QTDEVOLVIDA" => $newQtDevolvida,
                            "QTLIBERADA" => $receita['QTLIBERADA'],
                            "VRUNITRECEPRAT" => $valorUnit,
                            "VRTOTALRECEPRAT" => $valorTotal,
                            "IDTPRECEITA" => $receita['IDTPRECEITA'],
                            "IDADIANTADO" => $receita['IDADIANTADO'],
                            "NRSEQITRETI" => '',
                            "NRORG" => $nrorg,
                            "DTINCLUSAO" => $data,
                            "DTULTATU" => $data,
                            "NRORGINCLUSAO" => $nrorg,
                            "CDOPERINCLUSAO" => $operador,
                            "NRORGULTATU" => $nrorg,
                            "CDOPERULTATU" => $operador,
                            "IDATIVO" => 'S',
                            "IDCOOKCHILL" => $receita['IDCOOKCHILL'],
                            "IDFINALIZACAO" => $receita['IDFINALIZACAO'],
                            "IDTPPRODCHILL" => $receita['IDTPPRODCHILL'],
                            "IDESTADO" => $receita['IDESTADO'],
                            "NRSEQPRODACABADO" => $receita['NRSEQPRODACABADO'],
                            "OPERACAO" => 1,
                            "__is_new" => true
                        );
                    }
                    if (count($planRecProd) > 0) {
                        $linhaExclusao = array();
                        foreach ($planRecProd as $receitaExcl) {
                            $receitaExcl["OPERACAO"] = -1;
                            $linhaExclusao[] = $receitaExcl;
                        }
                        
                         $dataSet = new DataSet('planrecprod', $linhaExclusao);
                         $this->crudService->delete($response, $dataSet);
                        
                        //Valida a qtd total inserida e atualiza o ultimo se precisar
                        if (($qtProduto - $receita['QTLIB_CONC']) > 0) {
                            $ultimo = end($linhaNova);
                            array_pop($linhaNova);
                            $ultimo["QTAPRODUZIR"] += round($qtProduto - $receita['QTLIB_CONC'], 5);
                            $ultimo["QTDEVOLVIDA"] += round($qtInsDevolvida, 5);
                            $ultimo["QTEXTRA"] += round($qtInsExtra, 5);
                            $linhaNova[] = $ultimo;
                        }
                        
                        
                        $this->crudService->save('planrecprod', $linhaNova);
                        
                        // $this->atualizaCustoFiliaGreg(
                        //     $linhaNova
                        // );
                    }

                    $registrosAlterados++;
                }
            }
            $response->addDataset(new DataSet('planrecprod', array($registrosAlterados)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    private function getQtAproduzirConc($nrplanprod, $cdproduto){
        
        $connection = $this->entityManager->getConnection();
        $query = "SELECT NVL(SUM(R.QTAPRODUZIR), 0) AS QTAPRODUZIR_CONC
                    FROM PLANRECPROD R
                    INNER JOIN PLANPRODFILIAGREGA   F ON F.NRPLANPROD = R.NRPLANPROD
                                                       AND F.CDPRODACABADO = R.CDPRODACABADO
                                                       AND F.CDFILIAGREG = R.CDFILIAGREG
                                                       AND F.CDSERVFILIAGREG = R.CDSERVFILIAGREG
                                                       AND F.DTCONSUMO = R.DTCONSUMO
                                                       AND F.NRSEQPRODACABADO = R.NRSEQPRODACABADO
                                                       AND F.NRORG = R.NRORG
                                                       AND F.IDPRODUZIR = 'S'
                                                       AND F.IDCONCLUIDO = 'S'
                    WHERE R.NRPLANPROD =:NRPLANPROD
                    AND R.CDPRODUTO =:CDPRODUTO";
        $conditions = [
         "NRPLANPROD" => $nrplanprod,
         "CDPRODUTO" => $cdproduto 
        ]; 
        $data = $connection->fetchAll($query, $conditions);    
        return $data[0]['QTAPRODUZIR_CONC'];
    }
    
    private function atualizaCustoFiliaGreg($param){
        
         $vrTotalReceprat = $this->somaVrTotalReceprat($param[0]);
        
         $connection = $this->entityManager->getConnection();
         $query= "UPDATE PLANPRODFILIAGREGA SET VRTOTALPRODSERV = :VRTOTALPRODSERV , 
                                                VRUNITPRODSERV = ROUND(( :VRTOTALPRODSERV / QTCOMENSSERV), 2), 
                                               QTEXTRA = :QTEXTRA 
                                           WHERE (NRPLANPROD = :NRPLANPROD) AND (CDPRODACABADO = :CDPRODACABADO) AND (CDFILIAGREG = :CDFILIAGREG) AND (CDSERVFILIAGREG = :CDSERVFILIAGREG) AND (DTCONSUMO = :DTCONSUMO) AND (IDCOOKCHILL = :IDCOOKCHILL) AND (IDFINALIZACAO = :IDFINALIZACAO) 
                                           AND (IDTPPRODCHILL = :IDTPPRODCHILL) AND (IDESTADO = :IDESTADO) AND (NRSEQPRODACABADO = :NRSEQPRODACABADO)";
        
		foreach($param as $item) {
            $connection->executeQuery($query,
                array("VRTOTALPRODSERV"=> $vrTotalReceprat,"NRPLANPROD"=>$item['NRPLANPROD'],
                      "CDPRODACABADO"=>$item['CDPRODACABADO'],
                      "CDFILIAGREG"=>$item['CDFILIAGREG'],
                      "CDSERVFILIAGREG"=>$item['CDSERVFILIAGREG'], "DTCONSUMO"=>$item['DTCONSUMO'],
                      "IDCOOKCHILL"=>$item['IDCOOKCHILL'], "IDFINALIZACAO"=>$item['IDFINALIZACAO'],
                      "IDTPPRODCHILL"=>$item['IDTPPRODCHILL'], "IDESTADO"=>$item['IDESTADO'],
                      "NRSEQPRODACABADO"=>$item['NRSEQPRODACABADO'], "OPERACAO"=>$item['OPERACAO'],
                      "QTEXTRA"=> $item['QTEXTRA']
                )
            );
        }
    }
    
    private function somaVrTotalReceprat($param){
        
        $connection = $this->entityManager->getConnection();
        $query = "SELECT SUM(VRTOTALRECEPRAT) AS VRTOTALRECEPRAT FROM PLANRECPROD WHERE NRPLANPROD = :NRPLANPROD AND CDPRODACABADO = :CDPRODACABADO AND CDFILIAGREG = :CDFILIAGREG 
                  AND CDSERVFILIAGREG = :CDSERVFILIAGREG AND IDCOOKCHILL  = :IDCOOKCHILL AND IDTPPRODCHILL  = :IDTPPRODCHILL AND IDESTADO  = :IDESTADO 
                  AND NRSEQPRODACABADO  = :NRSEQPRODACABADO AND IDFINALIZACAO  = :IDFINALIZACAO AND NRORG  = :NRORG";
        $conditions = [
         "NRPLANPROD" => $param['NRPLANPROD'],
         "CDPRODACABADO" => $param['CDPRODACABADO'],
         "CDFILIAGREG"   => $param['CDFILIAGREG'],
         "CDSERVFILIAGREG"=> $param['CDSERVFILIAGREG'],
         "IDCOOKCHILL"  =>  $param['IDCOOKCHILL'],
         "IDTPPRODCHILL" => $param['IDTPPRODCHILL'],
         "IDESTADO" =>   $param['IDESTADO'],
         "NRSEQPRODACABADO" => $param['NRSEQPRODACABADO'],
         "IDFINALIZACAO"=> $param['IDFINALIZACAO'],
         "NRORG" => $param['NRORG']
        ]; 
        $data = $connection->fetchAll($query, $conditions);    
        return $data[0]['VRTOTALRECEPRAT'];
    }
    
    public function excluiLogMatprima(
        DTO\Request\Row $request, DTO\Response $response
    ){
        try {
            $row = $request->getRow();
            
            $nrorg = $this->crudService->getEnvironment()->getNrOrg();
            $nrlogmatprima = $row['NRLOGMATPRIMA'];

            $connection = $this->entityManager->getConnection();
            $query="DELETE FROM LOGMATPRIMA WHERE NRLOGMATPRIMA = :NRLOGMATPRIMA AND NRORG=:NRORG";
            $connection->executeQuery($query, array("NRLOGMATPRIMA"=>$nrlogmatprima, "NRORG"=>$nrorg));

            $response->addDataset(new DataSet('LOGMATPRIMA', array(true)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function salvaLiberada(DTO\Request\Row $request, DTO\Response $response){
        $rows = $request->getRow();
        $sql = "SELECT R.* FROM PLANPRODFILIAGREGA PA INNER JOIN PLANRECPROD R ON R.CDPRODACABADO = PA.CDPRODACABADO AND R.NRPLANPROD = PA.NRPLANPROD AND R.CDFILIAGREG = PA.CDFILIAGREG 
                AND R.CDSERVFILIAGREG = PA.CDSERVFILIAGREG AND R.DTCONSUMO = PA.DTCONSUMO AND R.IDCOOKCHILL = PA.IDCOOKCHILL AND R.IDTPPRODCHILL = PA.IDTPPRODCHILL AND R.IDESTADO = PA.IDESTADO 
                WHERE R.CDPRODUTO = :CDPRODUTO AND R.NRORG = :NRORG AND PA.NRPLANPROD = :NRPLANPROD AND PA.IDCONCLUIDO = 'N'";
        foreach($rows as $row){
            if(isset($row['QTLIBERADA_inline'])){
                $params = array(
                 "NRPLANPROD"   => $row['NRPLANPROD'],
                 "CDPRODUTO"    => $row['CDPRODUTO'],
                 "NRORG"        => $this->crudService->getEnvironment()->getNrOrg()
                );
                
                $data = $this->entityManager->getConnection()->fetchAll($sql, $params);
                if(sizeof($data) > 0){
                    //cálculo proporção
                    $qtLiberada  = $row['QTLIBERADA'];
                    $qtPlanejada = $row['QTPRODRECEPRAT'];
                    $proporcao = $qtLiberada/$qtPlanejada;
                    
                    for($i = 0;$i < sizeof($data);$i++){
                        $data[$i]['__is_new'] = false;
                        $data[$i]['changed'] = true;
                        $data[$i]['QTLIBERADA'] = round($data[$i]['QTPRODRECEPRAT'] * $proporcao, 5) ;
                        $data[$i]['VRTOTALRECEPRAT'] =  round($data[$i]['QTLIBERADA'] * $data[$i]['VRUNITRECEPRAT'],5);
                    }
                    
                    $dataSourceName = "planrecprod";
                    $dataSet = new DataSet($dataSourceName, $data);
                    $this->dataSourceManager->persist($dataSet);
                
                }else{
                    //Produtos ja concluidos
                    $message = new Message($row['NMPRODUTO'], 'W');
                    $response->addMessage($message);
                } 
            }
        }
        
        
        
     }
   
    public function excluiEtiquetasProd(DTO\Request\Row $request, DTO\Response $response){
        $rows = $request->getRow();
        try {
            $dataSourceName = 'etiquetacoz';
            $connection = $this->entityManager->getConnection();
            $query="SELECT E.* FROM ETIQUETACOZ E INNER JOIN CARDACLIPLANPROD CP ON CP.NRCARDACLIPLANPROD = E.NRCARDACLIPLANPROD 
                                                    AND CP.NRPLANPROD = :NRPLANPROD AND CP.CDPRODUTO = :CDPRODUTO 
                                                    AND CP.CDPRATO = :CDPRATO AND CP.CDFILAUXPRAT = :CDFILAUXPRAT
                                                    AND CP.IDCOOKCHILL = :IDCOOKCHILL AND CP.IDTPPRODCHILL = :IDTPPRODCHILL
                                                    AND CP.IDESTADO = :IDESTADO";
            foreach ($rows as $row) {
                $params = array (
                    "NRPLANPROD"    => $row['NRPLANPROD'],
                    "CDPRODUTO"     => $row['CDPRODUTO'],
                    "CDPRATO"       => $row['CDPRATO'],
                    "CDFILAUXPRAT"  => $row['CDFILAUXPRAT'],
                    "IDCOOKCHILL"   => $row['IDCOOKCHILL'],
                    "IDTPPRODCHILL" => $row['IDTPPRODCHILL'],
                    "IDESTADO"      => $row['IDESTADO']
                );
                $data = $connection->fetchAll($query, $params);
                
                if(!empty($data)){
                    foreach ($data as $row) {
                        $newRequest = new DTO\Request\Row($row,"","","");
                        $rowToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
                     	$dataSet = new DataSet($dataSourceName, $rowToDelete);
                        $this->dataSourceManager->delete($dataSet);
                    }
                }
                
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
        
        
    } 
    
    public function excluiEtiquetasProdServ(DTO\Request\Row $request, DTO\Response $response){
        $rows = $request->getRow();
        
        try {
            $dataSourceName = 'etiquetacoz';
            $connection = $this->entityManager->getConnection();
            $query="SELECT E.* FROM ETIQUETACOZ E 
                    INNER JOIN CARDACLIPLANPROD CP ON CP.NRCARDACLIPLANPROD = E.NRCARDACLIPLANPROD
                                                AND CP.NRPLANPROD =:NRPLANPROD AND CP.CDPRODUTO =:CDPRODUTO
                                                AND CP.CDPRATO =:CDPRATO AND CP.CDFILAUXPRAT =:CDFILAUXPRAT
                                                AND CP.IDCOOKCHILL =:IDCOOKCHILL AND CP.IDTPPRODCHILL =:IDTPPRODCHILL
                                                AND CP.IDESTADO =:IDESTADO AND CP.CDFILIAL =:CDFILIAL
                                                AND CP.CDSERVFILI =:CDSERVFILI";
        
            foreach ($rows as $row) {
                $params = array (
                    "NRPLANPROD"    => $row['NRPLANPROD'],
                    "CDPRODUTO"     => $row['CDPRODUTO'],
                    "CDPRATO"       => $row['CDPRATO'],
                    "CDFILAUXPRAT"  => $row['CDFILAUXPRAT'],
                    "IDCOOKCHILL"   => $row['IDCOOKCHILL'],
                    "IDTPPRODCHILL" => $row['IDTPPRODCHILL'],
                    "IDESTADO"      => $row['IDESTADO'],
                    "CDFILIAL"      => $row['CDFILIAGREG'],
                    "CDSERVFILI"    => $row['CDSERVFILIAGREG']
               
                );
                $data = $connection->fetchAll($query, $params);
                
                if(!empty($data)){
                     foreach ($data as $row) {
                        $newRequest = new DTO\Request\Row($row,"","","");
                        $rowToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
                     	$dataSet = new DataSet($dataSourceName, $rowToDelete);
                        $this->dataSourceManager->delete($dataSet);
                         
                     }
                }    
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
        
        
    } 
    
    private function totalProdReceprat($planRecProd){
        $totalProdReceprat = 0;
        foreach($planRecProd as $receita){
            $totalProdReceprat += $receita['QTPRODRECEPRAT']; 
        }
        return $totalProdReceprat; 
    }
    
     public function validaQtMatPrima ( DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        try {
            $qtAprodConc = $this->getQtAproduzirConc($row['NRPLANPROD'], $row['CDPRODUTO']);
            if($row['QTAPRODUZIR'] < $qtAprodConc){
                $message = new Message(
                    'O total da quantidade consumida não pode ser menor que a quantidade já produzida!', 'M'
                );
                $response->addMessage($message);
            }
            
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
        
    }

}